@@ -11,6 +11,7 @@
 **                                                                                    
 ** -------------------------------------------------------------------------*/
 
+#pragma once
 
 #include "RTSPServer.hh"
 #include "RTSPCommon.hh"
@@ -116,7 +117,7 @@ class TCPSink: public MediaSink {
 #if LIVEMEDIA_LIBRARY_VERSION_INT < 1606435200
 #define SOCKETCLIENT sockaddr_in 
 #else
-#define SOCKETCLIENT sockaddr_storage 
+#define SOCKETCLIENT sockaddr_storage const&
 #endif
 class HTTPServer : public RTSPServer
 {
@@ -124,8 +125,12 @@ class HTTPServer : public RTSPServer
 	class HTTPClientConnection : public RTSPServer::RTSPClientConnection
 	{
 		public:
-			HTTPClientConnection(RTSPServer& ourServer, int clientSocket, struct SOCKETCLIENT clientAddr)
+			HTTPClientConnection(RTSPServer& ourServer, int clientSocket, struct SOCKETCLIENT clientAddr, Boolean useTLS)
+#if LIVEMEDIA_LIBRARY_VERSION_INT >= 1642723200      
+		       : RTSPServer::RTSPClientConnection(ourServer, clientSocket, clientAddr, useTLS), m_TCPSink(NULL), m_StreamToken(NULL), m_Subsession(NULL), m_Source(NULL) {
+#else
 		       : RTSPServer::RTSPClientConnection(ourServer, clientSocket, clientAddr), m_TCPSink(NULL), m_StreamToken(NULL), m_Subsession(NULL), m_Source(NULL) {
+#endif				   
 			}
 			virtual ~HTTPClientConnection();
 
@@ -150,8 +155,14 @@ class HTTPServer : public RTSPServer
 			FramedSource*          m_Source;
 	};
 	
+	class HTTPClientSession : public RTSPServer::RTSPClientSession {
+		public:
+			HTTPClientSession(HTTPServer& ourServer, u_int32_t sessionId) : RTSPServer::RTSPClientSession(ourServer, sessionId)  {}
+			virtual void handleCmd_SETUP(RTSPServer::RTSPClientConnection* ourClientConnection, char const* urlPreSuffix, char const* urlSuffix, char const* fullRequestStr);
+	};
+
 	public:
-		static HTTPServer* createNew(UsageEnvironment& env, Port rtspPort, UserAuthenticationDatabase* authDatabase, unsigned reclamationTestSeconds, unsigned int hlsSegment, const std::string webroot) 
+		static HTTPServer* createNew(UsageEnvironment& env, Port rtspPort, UserAuthenticationDatabase* authDatabase, unsigned reclamationTestSeconds, unsigned int hlsSegment, const std::string webroot, const char* sslCert) 
 		{
 			HTTPServer* httpServer = NULL;
 #if LIVEMEDIA_LIBRARY_VERSION_INT < 1610928000
@@ -167,26 +178,43 @@ class HTTPServer : public RTSPServer
 
 			if (ourSocketIPv4 != -1) 
 			{
-				httpServer = new HTTPServer(env, ourSocketIPv4, ourSocketIPv6, rtspPort, authDatabase, reclamationTestSeconds, hlsSegment, webroot);
+				httpServer = new HTTPServer(env, ourSocketIPv4, ourSocketIPv6, rtspPort, authDatabase, reclamationTestSeconds, hlsSegment, webroot, sslCert);
 			}
 			return httpServer;
 		}
 
-		HTTPServer(UsageEnvironment& env, int ourSocketIPv4, int ourSocketIPv6, Port rtspPort, UserAuthenticationDatabase* authDatabase, unsigned reclamationTestSeconds, unsigned int hlsSegment, const std::string & webroot)
-		  : RTSPServer(env, ourSocketIPv4, ourSocketIPv6, rtspPort, authDatabase, reclamationTestSeconds), m_hlsSegment(hlsSegment), m_webroot(webroot)
+#if LIVEMEDIA_LIBRARY_VERSION_INT	<	1611187200
+		HTTPServer(UsageEnvironment& env, int ourSocketIPv4, int ourSocketIPv6, Port rtspPort, UserAuthenticationDatabase* authDatabase, unsigned reclamationTestSeconds, unsigned int hlsSegment, const std::string & webroot, const char* sslCert)
+		  : RTSPServer(env, ourSocketIPv4, rtspPort, authDatabase, reclamationTestSeconds), m_hlsSegment(hlsSegment), m_webroot(webroot), m_sslCert(sslCert)
+#else
+		HTTPServer(UsageEnvironment& env, int ourSocketIPv4, int ourSocketIPv6, Port rtspPort, UserAuthenticationDatabase* authDatabase, unsigned reclamationTestSeconds, unsigned int hlsSegment, const std::string & webroot, const char* sslCert)
+		  : RTSPServer(env, ourSocketIPv4, ourSocketIPv6, rtspPort, authDatabase, reclamationTestSeconds), m_hlsSegment(hlsSegment), m_webroot(webroot), m_sslCert(sslCert)
+#endif			
 		{
                        if ( (!m_webroot.empty()) && (*m_webroot.rend() != '/') ) {
                                m_webroot += "/";
                        }
+#if LIVEMEDIA_LIBRARY_VERSION_INT >= 1642723200      
+                if (this->isSSL()) {
+                    this->setTLSState(m_sslCert, m_sslCert, true, true);
+                }
+#endif   			
 		}
 
-		RTSPServer::RTSPClientConnection* createNewClientConnection(int clientSocket, struct SOCKETCLIENT clientAddr) 
+		virtual RTSPServer::ClientConnection* createNewClientConnection(int clientSocket, struct SOCKETCLIENT clientAddr) 
 		{
-			return new HTTPClientConnection(*this, clientSocket, clientAddr);
+			return new HTTPClientConnection(*this, clientSocket, clientAddr, this->isSSL());
 		}
 		
+		virtual RTSPServer::ClientSession* createNewClientSession(u_int32_t sessionId) {
+			return new HTTPClientSession(*this, sessionId);
+		}
+
+		bool isSSL() { return (m_sslCert != NULL); }
+
         private:
-		const unsigned int m_hlsSegment;
-		std::string  m_webroot;
+			const unsigned int m_hlsSegment;
+			std::string  m_webroot;
+			const char*  m_sslCert;
 };
 
