@@ -247,15 +247,14 @@ void HTTPServer::HTTPClientConnection::handleHTTPCmd_StreamingGET(char const* ur
 	else if (strncmp(urlSuffix, "getStreamList", strlen("getStreamList")) == 0) 
 	{
 		std::ostringstream os;
-		HTTPServer* httpServer = (HTTPServer*)(&fOurServer);
-		ServerMediaSessionIterator it(*httpServer);
-		ServerMediaSession* serverSession = NULL;
 		if (questionMarkPos != NULL) {
 			questionMarkPos++;
 			os << "var " << questionMarkPos << "=";
 		}
 		os << "[\n";
 		bool first = true;
+		ServerMediaSessionIterator it(fOurServer);
+		ServerMediaSession* serverSession = NULL;
 		while ( (serverSession = it.next()) != NULL) {
 			if (serverSession->duration() > 0) {
 				if (first) 
@@ -343,7 +342,12 @@ void HTTPServer::HTTPClientConnection::handleHTTPCmd_StreamingGET(char const* ur
 		sockaddr_storage clientAddress = { 0 };
 		sockaddr_storage destinationAddress = { 0 };
 #endif		
+#if LIVEMEDIA_LIBRARY_VERSION_INT < 1636848000		
 		subsession->getStreamParameters(m_ClientSessionId, clientAddress, clientRTPPort, clientRTCPPort, -1, 0, 0, destinationAddress, destinationTTL, isMulticast, serverRTPPort, serverRTCPPort, m_StreamToken);
+#else
+
+		subsession->getStreamParameters(m_ClientSessionId, clientAddress, clientRTPPort, clientRTCPPort, -1, 0, 0, NULL, destinationAddress, destinationTTL, isMulticast, serverRTPPort, serverRTCPPort, m_StreamToken);
+#endif		
 
 		// Seek the stream source to the desired place, with the desired duration, and (as a side effect) get the number of bytes:
 		double dOffsetInSeconds = (double)offsetInSeconds;
@@ -370,10 +374,14 @@ void HTTPServer::HTTPClientConnection::handleHTTPCmd_StreamingGET(char const* ur
 	} 
 }
 
+void HTTPServer::HTTPClientSession::handleCmd_SETUP(RTSPServer::RTSPClientConnection* ourClientConnection, char const* urlPreSuffix, char const* urlSuffix, char const* fullRequestStr) {
+	envir() << "handleCmd_SETUP:" << fullRequestStr;
+	RTSPServer::RTSPClientSession::handleCmd_SETUP(ourClientConnection, urlPreSuffix, urlSuffix, fullRequestStr);
+}
+
 void HTTPServer::HTTPClientConnection::handleCmd_notFound() {
 	std::ostringstream os;
-	HTTPServer* httpServer = (HTTPServer*)(&fOurServer);
-	ServerMediaSessionIterator it(*httpServer);
+	ServerMediaSessionIterator it(fOurServer);
 	ServerMediaSession* serverSession = NULL;
 	while ( (serverSession = it.next()) != NULL) {
 		os << serverSession->streamName() << "\n";
